package immibis.infinitubes;


import immibis.core.api.porting.PortableBlockRenderer;
import immibis.core.api.util.Dir;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.client.ForgeHooksClient;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public class InfiniTubeStaticRenderer implements PortableBlockRenderer {
	private float u, v;
	private RenderBlocks render;
	private void setTexture(int i) {
		if(render.overrideBlockTexture >= 0)
			i = render.overrideBlockTexture;
		u = (i % 16) / 16.0f;
		v = (i / 16) / 16.0f;
	}
	private RotatedTessellator rt = new RotatedTessellator();
	
	public static final InfiniTubeStaticRenderer INSTANCE = new InfiniTubeStaticRenderer();
	
	private void renderLANWire(int tex, int x, int y, int z) {
		setTexture(tex);
		
		immibis.core.RenderUtils.setBrightness(render.blockAccess, x, y, z);
		Tessellator.instance.setColorRGBA(255, 255, 255, 255);
		
		boolean nx = InfiniTubeBlock.connects(render.blockAccess, x-1,y,z, Dir.NX);
		boolean px = InfiniTubeBlock.connects(render.blockAccess, x+1,y,z, Dir.PX);
		boolean ny = InfiniTubeBlock.connects(render.blockAccess, x,y-1,z, Dir.NY);
		boolean py = InfiniTubeBlock.connects(render.blockAccess, x,y+1,z, Dir.PY);
		boolean nz = InfiniTubeBlock.connects(render.blockAccess, x,y,z-1, Dir.NZ);
		boolean pz = InfiniTubeBlock.connects(render.blockAccess, x,y,z+1, Dir.PZ);
		
		final double min = 4/16f, max=12/16f;
		
		//System.out.println("render "+x+" "+y+" "+z+" "+nx+" "+px+" "+ny+" "+py+" "+nz+" "+pz);
		
		if(!nx && !ny && !nz && !px && !py && !pz) {
			rt.setup(x, y, z, Dir.NY);
			renderLANWireVertical(min, max, true, true);
			return;
		}
		
		if(nx || px) {
			rt.setup(x, y, z, Dir.NX);
			renderLANWireVertical(nx?0:min, px?1:max, !nx && !ny && !py && !nz && !pz, !px && !ny && !py && !nz && !pz);
		}
		
		if(ny || py) {
			rt.setup(x, y, z, Dir.NY);
			renderLANWireVertical(ny?0:min, py?1:max, !ny && !px && !px && !nz && !pz, !py && !nx && !px && !nz && !pz);
		}
		
		if(nz || pz) {
			rt.setup(x, y, z, Dir.NZ);
			renderLANWireVertical(nz?0:min, pz?1:max, !nz && !ny && !py && !nx && !px, !pz && !ny && !py && !nx && !px);
		}
	}
	
	private void renderLANWireVertical(double bottom, double top, boolean bottomcap, boolean topcap) {
		final double min = 4/16.0, max=12/16.0, du=8/256.0, dvt=top/16.0, dvb=bottom/16.0;
		
		topcap = true;
		bottomcap = true;
		
		rt.setNormal(0, 0, -1);
		rt.addVertexWithUV(min, top, min, u, v+dvt);
		rt.addVertexWithUV(max, top, min, u+du, v+dvt);
		rt.addVertexWithUV(max, bottom, min, u+du, v+dvb);
		rt.addVertexWithUV(min, bottom, min, u, v+dvb);
		
		rt.setNormal(0, 0, 1);
		rt.addVertexWithUV(min, bottom, max, u, v+dvb);
		rt.addVertexWithUV(max, bottom, max, u+du, v+dvb);
		rt.addVertexWithUV(max, top, max, u+du, v+dvt);
		rt.addVertexWithUV(min, top, max, u, v+dvt);
		
		rt.setNormal(-1, 0, 0);
		rt.addVertexWithUV(min, bottom, min, u, v+dvb);
		rt.addVertexWithUV(min, bottom, max, u+du, v+dvb);
		rt.addVertexWithUV(min, top, max, u+du, v+dvt);
		rt.addVertexWithUV(min, top, min, u, v+dvt);
		
		rt.setNormal(1, 0, 0);
		rt.addVertexWithUV(max, top, min, u, v+dvt);
		rt.addVertexWithUV(max, top, max, u+du, v+dvt);
		rt.addVertexWithUV(max, bottom, max, u+du, v+dvb);
		rt.addVertexWithUV(max, bottom, min, u, v+dvb);
		
		if(bottomcap) {
			double d1 = (4/256.0), d2=(12/256.0);
			
			rt.setNormal(-1, 0, 0);
			rt.addVertexWithUV(max, bottom, min, u+du, v+d1);
			rt.addVertexWithUV(max, bottom, max, u+du, v+d2);
			rt.addVertexWithUV(min, bottom, max, u, v+d2);
			rt.addVertexWithUV(min, bottom, min, u, v+d1);
			
		}
		
		if(topcap) {
			double d1 = (4/256.0), d2=(12/256.0);
			
			rt.setNormal(1, 0, 0);
			rt.addVertexWithUV(min, top, min, u, v+d1);
			rt.addVertexWithUV(min, top, max, u, v+d2);
			rt.addVertexWithUV(max, top, max, u+du, v+d2);
			rt.addVertexWithUV(max, top, min, u+du, v+d1);
			
		}
	}

	@Override
	public boolean renderWorldBlock(RenderBlocks render, IBlockAccess world, int x, int y, int z, Block block, int model) {
		this.render = render;
		if(render.overrideBlockTexture < 0)
			ForgeHooksClient.bindTexture("/immibis/infinitubes/world.png", 0);
		renderLANWire(0, x, y, z);
		if(render.overrideBlockTexture < 0)
			ForgeHooksClient.unbindTexture();
		return true;
	}

	@Override
	public void renderInvBlock(RenderBlocks render, Block block, int meta, int model) {
		this.render = render;
		rt.setup(-0.5, -0.5, -0.5, Dir.NY);
		
		setTexture(0);
		
		ForgeHooksClient.bindTexture("/immibis/infinitubes/world.png", 0);
		
		rt.base.startDrawingQuads();
		rt.base.setColorOpaque(255, 255, 255);
		renderLANWireVertical(0, 1, true, true);
		rt.base.draw();
		
		ForgeHooksClient.unbindTexture();
		
	}
}
